#include <windows.h>
#include <GL/glew.h>
#include <GL/wglew.h>
#include <assert.h>
#include <vector>

static HWND hwnd = NULL;
static HGLRC ctx = NULL;

static bool init()
{
	HINSTANCE hinstance = (HINSTANCE)GetModuleHandle(NULL);

	WNDCLASSA window_class;
	window_class.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC | CS_GLOBALCLASS;
	window_class.lpfnWndProc = DefWindowProc;
	window_class.cbClsExtra = 0;
	window_class.cbWndExtra = 0;
	window_class.hInstance = hinstance;
	window_class.hIcon = NULL;
	window_class.hCursor = LoadCursor (NULL, IDC_ARROW);
	window_class.hbrBackground = (HBRUSH)GetStockObject (WHITE_BRUSH);
	window_class.lpszMenuName = NULL;
	window_class.lpszClassName = "test_class";

	ATOM atom = RegisterClassA(&window_class);
	hwnd = CreateWindowA ("test_class", "htest", WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, 1,1,1,1,	NULL, NULL, hinstance, NULL);

	if (hwnd == NULL) {
		DWORD err = GetLastError();	
		return false;
	}

	HDC hDC = GetDC (hwnd);

	if (hDC == NULL) {
		return false;
	}

	PIXELFORMATDESCRIPTOR const pfd = 
	{
		sizeof (PIXELFORMATDESCRIPTOR),
		1,						
		PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_TYPE_RGBA,		
		0,					
		0, 0, 0, 0, 0, 0,	
		0,					
		0,					
		0,					
		0, 0, 0, 0,			
		0,					
		0,					
		0,					
		PFD_MAIN_PLANE,		
		0,					
		0, 0, 0				
	};
	int pixel_format = ChoosePixelFormat(hDC, &pfd);
	SetPixelFormat (hDC, pixel_format, &pfd);
	ctx = wglCreateContext (hDC);

	if (ctx) {		
		if (!wglMakeCurrent(hDC,ctx)) {
			return false;
		}
	}

	ReleaseDC (hwnd, hDC);

	return true;
}

static void check_gl_error()
{
	GLenum error = glGetError();

	assert(error == GL_NO_ERROR);
}

int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	bool res = init();

	GLenum glew = glewInit();

	if (wglGetGPUIDsAMD && wglGetGPUInfoAMD)
	{	
#if 1
		UINT n = wglGetGPUIDsAMD(0, 0);
		UINT *	ids = new UINT[n];
		UINT	total_mem_mb = 0;
		wglGetGPUIDsAMD(n, ids);
		wglGetGPUInfoAMD(ids[0], WGL_GPU_RAM_AMD, GL_UNSIGNED_INT, sizeof(UINT), &total_mem_mb);
		delete[] ids;
#else
		// Supposedly working code from http://devgurus.amd.com/thread/159595

		// non sense call  
		wglGetGPUIDsAMD(0, 0);  
  
		// retrieve the id of the device from the current context  
		UINT gpuId = wglGetContextGPUIDAMD(ctx);  
  
		// retrieve the amount of ram  
		UINT gpuMem = 0;     
		wglGetGPUInfoAMD(gpuId, WGL_GPU_RAM_AMD, GL_UNSIGNED_INT, sizeof(UINT), &gpuMem); 
#endif

	}

	return 0;
}
